% This Matlab script plots a xy-plane from a 3D-output binary files generated by NS3D (velo_rho_vort.t=xxx.xxxx)


% ############################################
% Settings to modify directly in this script
% ############################################

% We choose the plotting Matlab function
%   1) "contour" function. Nice but long.
%   2) "pcolor" function. Quick but less nice.
plotting_function = 2;


% ##########################
% No more modification !!
% ##########################

fprintf(1, '\nThis Matlab script plots a xy-plane from a 3D-output binary files generated by NS3D\n');


% We ask for the data file to plot
dir
name_file = input('\nOutput data file to plot: ', 's');


% We determine whether the data file is coded in little-endian or in big-endian
% we try to read the file in little endian (PC)
type_endian = 'ieee-le';
FID = fopen(name_file, 'r',type_endian);
poub = fread(FID, 1, 'integer*4');
flag = fread(FID, 1, 'integer*4');
status = fclose(FID);

% if the flag is different from 1, we try to read the file in big-endian (SGI, NEC, IBM)
if flag ~= 1
    type_endian = 'ieee-be';
    FID = fopen(name_file, 'r',type_endian);
    poub = fread(FID, 1, 'integer*4');
    flag = fread(FID, 1, 'integer*4');
    status = fclose(FID);

    % we check again whether the flag is 1
    if flag ~= 1
        fprintf(1, '\n The file format is not correct: the header flag is different from 1\n');
        return
    end
end


% we read all the data stored in the header
FID = fopen(name_file, 'r',type_endian);

poub            = fread(FID, 1, 'integer*4');
flag            = fread(FID, 1, 'integer*4');
nx              = fread(FID, 1, 'integer*4');
ny              = fread(FID, 1, 'integer*4');
nz              = fread(FID, 1, 'integer*4');
lx              = fread(FID, 1, 'real*8');
ly              = fread(FID, 1, 'real*8');
lz              = fread(FID, 1, 'real*8');
dt              = fread(FID, 1, 'real*8');
truncate        = fread(FID, 1, 'integer*4');
type_trunc      = fread(FID, 1, 'integer*4');
rtrunc_x        = fread(FID, 1, 'real*8');
rtrunc_y        = fread(FID, 1, 'real*8');
rtrunc_z        = fread(FID, 1, 'real*8');
nu              = fread(FID, 1, 'real*8');
stratification  = fread(FID, 1, 'integer*4');
xns             = fread(FID, 1, 'real*8');
schm            = fread(FID, 1, 'real*8');
omega2          = fread(FID, 1, 'real*8');
perturb         = fread(FID, 1, 'integer*4');
lin             = fread(FID, 1, 'integer*4');
poub            = fread(FID, 1, 'integer*4');

poub            = fread(FID, 1, 'integer*4');
instant         = fread(FID, 1, 'real*8');
status = fclose(FID);


% we print basic informations about the simulation
fprintf(1, '\n    Dimensions of the simulation: %i * %i * %i \n', nx, ny, nz);
fprintf(1, '    Dimensions of the computational box: %f * %f * %f \n', lx, ly, lz);

if (stratification == 0)
    freq_brunt = 0;
else
    freq_brunt = xns;
end

fprintf(1, '    Stratification: %f \n', freq_brunt);

if (perturb==0)
    texte = ['non perturbative'];
else
    if (lin==0) then
        texte = ['perturbative/non linear'];
    else
        texte = ['perturbative/linear'];
    end
end


% we define the collocation grid
X = linspace(0, lx-lx/nx, nx);
Y = linspace(0, ly-ly/ny, ny);
Z = linspace(0, lz-lz/nz, nz);

% we ask which field to plot
% field to plot (vx=1,vy=2,vz=3,rho=4,wx=5,wy=6,wz=7)
type_field=floor(input('\nfield to plot (vx=1,vy=2,vz=3,rho=4,wx=5,wy=6,wz=7): '));

% we ask the user which plane cut to plot
iz = floor(input(['\n z-plane-index (0-' num2str(nz-1) '):']));

% we read the corresponding data
FID = fopen(name_file, 'r',type_endian);
fseek(FID,33*4 + (4+8+4) + (type_field-1)*(4+8*nx*ny*nz+4) + 4 + 8*(iz-1)*nx*ny, 'bof');
Vort_Z_iz = fread(FID, nx*ny, 'real*8');
status = fclose(FID);

Vort_Z_iz = reshape(Vort_Z_iz, [nx,ny]);

% we plot the data
figure
if plotting_function==1
    contour(X,Y,Vort_Z_iz',20);
    colorbar;
else
    pcolor(X,Y,Vort_Z_iz');
    colorbar;
    axis equal tight
	shading interp
	lighting phong
end

xlabel('x');
ylabel('y');
axis equal tight;
title({['Field  t=', num2str(instant)]; ['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  dt=',num2str(dt), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte]},'Interpreter','none');
