% This Matlab script plots a 3D view from a 3D-output binary files generated by NS3D (velo_rho_vort.t=xxx.xxxx)

% ######################
% Parameters that must be filled directly in this script
% ######################

% Strides along each direction. This is useful to save memory and speedup plotting with large data files.
% For instance str_x=2 means that we read only 1 point out of 2, along the x-direction, in the output data binary file.
% A value str_x=1 means that all the data are read and plotted.
str_x=1;
str_y=1;
str_z=1;

% Do we save the figure .fig in a directory 3D_views ?
save_file=0;

% Do we plot an additional plane cut? If so, we give the x-position of this plane cut
cut_plane=0;
x_coupe = 2.5;

% Do we clip the plotting box, to zoom on an area of interest? If so, we define the area of interest.
% The clipping is only active on the xy-plane. The full z-direction is always plotted.
clipping=0;
x_cl=0;
x_cu=4.2;
y_cl=1.;
y_cu=4.2;


% ######################
% No more modification !!
% ######################

fprintf(1, '\nThis Matlab script plots a 3D view from a 3D-output binary files generated by NS3D\n');


% We ask for the data file to plot
dir
fichier = input('\nOutput data file to plot: ', 's');


% We determine whether the data file is coded in little-endian or in big-endian
% we try to read the file in little endian (PC)
type_endian = 'ieee-le';
FID = fopen(name_file, 'r',type_endian);
poub = fread(FID, 1, 'integer*4');
flag = fread(FID, 1, 'integer*4');
status = fclose(FID);

% If the flag is different from 1, we try to read the file in big-endian (SGI, NEC, IBM)
if flag ~= 1
    type_endian = 'ieee-be';
    FID = fopen(name_file, 'r',type_endian);
    poub = fread(FID, 1, 'integer*4');
    flag = fread(FID, 1, 'integer*4');
    status = fclose(FID);

    % we check again whether the flag is 1
    if flag ~= 1
        fprintf(1, '\n The file format is not correct: the header flag is different from 1\n');
        return
    end
end


% We read all the data stored in the header
FID = fopen(fichier, 'r',type_endian);

poub            = fread(FID, 1, 'integer*4');
flag            = fread(FID, 1, 'integer*4');
nx              = fread(FID, 1, 'integer*4');
ny              = fread(FID, 1, 'integer*4');
nz              = fread(FID, 1, 'integer*4');
lx              = fread(FID, 1, 'real*8');
ly              = fread(FID, 1, 'real*8');
lz              = fread(FID, 1, 'real*8');
dt              = fread(FID, 1, 'real*8');
truncate        = fread(FID, 1, 'integer*4');
type_trunc      = fread(FID, 1, 'integer*4');
rtrunc_x        = fread(FID, 1, 'real*8');
rtrunc_y        = fread(FID, 1, 'real*8');
rtrunc_z        = fread(FID, 1, 'real*8');
nu              = fread(FID, 1, 'real*8');
stratification  = fread(FID, 1, 'integer*4');
xns             = fread(FID, 1, 'real*8');
schm            = fread(FID, 1, 'real*8');
omega2          = fread(FID, 1, 'real*8');
perturb         = fread(FID, 1, 'integer*4');
lin             = fread(FID, 1, 'integer*4');
poub            = fread(FID, 1, 'integer*4');

status = fclose(FID);

% We print basic informations about the simulation
fprintf(1, '\n    Dimensions of the simulation: %i * %i * %i \n', nx, ny, nz);
fprintf(1, '    Dimensions of the computational box: %f * %f * %f \n', lx, ly, lz);

if (stratification == 0)
    freq_brunt = 0;
else
    freq_brunt = xns;
end

fprintf(1, '    Stratification: %f \n', freq_brunt);

if (perturb==0)
    texte = ['non perturbative'];
else
    if (lin==0) then
        texte = ['perturbative/non linear'];
    else
        texte = ['perturbative/linear'];
    end
end

% We determine the number of points to plot along each direction, taking the strides into account
nxp = 1+floor( (nx-1)/str_x);
nyp = 1+floor( (ny-1)/str_y);
nzp = 1+floor( (nz-1)/str_z);

fprintf(1, '\n Dimensions used for the plot: %i * %i * %i \n', nxp, nyp, nzp);


% We ask which field to plot
% field to plot (vx=1,vy=2,vz=3,rho=4,wx=5,wy=6,wz=7)
type_field=floor(input('\nfield to plot (vx=1,vy=2,vz=3,rho=4,wx=5,wy=6,wz=7): '));


% We define the collocation grid
X = linspace(0, lx-lx/nxp, nxp);
Y = linspace(0, ly-ly/nyp, nyp);
Z = linspace(0, lz-lz/nzp, nzp);

scrsz = get(0,'ScreenSize');
figure('Position',[100 100 scrsz(3)*0.8 scrsz(4)*0.8])

fprintf(1, '      Reading...');
FID = fopen(fichier,'r',type_endian);

% We read all the header at once
fseek(FID, 4*34, 'bof');
instant = fread(FID, 1, 'real*8');

% We read the data
% no stride: classical reading
if (str_x==1 & str_y==1 & str_z==1)
	fseek(FID, 33*4 + (4+8+4) + (type_field-1)*(4+8*nx*ny*nz+4) + 4, 'bof');
	field = fread(FID, nxp*nyp*nzp, 'real*8');
	field = reshape(field, [nxp,nyp,nzp]);
% there is a stride: more tricky
else
	field = zeros(nxp,nyp,nzp);
	for izp=1:nzp
		for iyp=1:nyp
            fseek(FID,33*4 + (4+8+4) + (type_field-1)*(4+8*nx*ny*nz+4) + 4 + 8*(izp-1)*str_z*nx*ny + 8*(iyp-1)*str_y*nx, 'bof');
            field(:,iyp,izp) = fread(FID, nxp, 'real*8', 8*(str_x-1));
		end
    end
end

status = fclose(FID);

% we plot the cut plane
if (cut_plane==1)
    hold on
    nxl = floor(nxp/4);
    nxu = floor(3*nxp/4);
    nyl = floor(nyp/4);
    nyu = floor(3*nyp/4);
    
    [xx,yy,zz] = meshgrid(X(nxl:nxu),Y(nyl:nyu),Z);
    xslice = [x_coupe]; yslice = []; zslice = [];
    slice(xx,yy,zz,field(nxl:nxu,nyl:nyu,:),xslice,yslice,zslice)
    shading interp
end


% we clip the field to the area of interest
if (clipping==1)
    nx_cl=max(floor(x_cl/4.2*nxp),1);
    nx_cu=min(floor(x_cu/4.2*nxp),nxp);

    ny_cl=max(floor(x_cl/4.2*nyp),1);
    ny_cu=min(floor(x_cu/4.2*nyp),nyp);

    field = field(nx_cl:nx_cu,ny_cl:ny_cu,:);
else
    nx_cl=1;
    nx_cu=nxp;

    ny_cl=1;;
    ny_cu=nyp;
end

% Plotting
fprintf(1, '      Plotting...\n');

pmoy_v=mean(max(max(field)));
nmoy_v=-pmoy_v;

p = patch(isosurface(Y(ny_cl:ny_cu),X(nx_cl:nx_cu),Z,field,0.6*pmoy_v));
isonormals(Y(ny_cl:ny_cu),X(nx_cl:nx_cu),Z,field,p)
set(p,'FaceAlpha',0.9,'FaceColor','red','EdgeColor','none');
p2 = patch(isosurface(Y(ny_cl:ny_cu),X(nx_cl:nx_cu),Z,field,0.1*pmoy_v));
isonormals(Y(ny_cl:ny_cu),X(nx_cl:nx_cu),Z,field,p2)
set(p2,'FaceAlpha',0.15,'FaceColor',[255,251,140]/256,'EdgeColor','none');

p3 = patch(isosurface(Y(ny_cl:ny_cu),X(nx_cl:nx_cu),Z,field,0.6*nmoy_v));
isonormals(Y(ny_cl:ny_cu),X(nx_cl:nx_cu),Z,field,p3)
set(p3,'FaceAlpha',0.9,'FaceColor','blue','EdgeColor','none');
p4 = patch(isosurface(Y(ny_cl:ny_cu),X(nx_cl:nx_cu),Z,field,0.1*nmoy_v));
isonormals(Y(ny_cl:ny_cu),X(nx_cl:nx_cu),Z,field,p4)
set(p4,'FaceAlpha',0.15,'FaceColor',[130,186,212]/256,'EdgeColor','none');

daspect([1 1 1])
view(3);
axis([Y(1) Y(end) X(1) X(end) Z(1) Z(end)])
camlight
lighting gouraud
axis on
box off
set(gca,'Visible','on')


% we save the figure in a .fig file in a directory "3D_views"
if (save_file==1)
    !mkdir -p 3D_views
    saveas(gcf,['3D_views/3D_view.t=',num2str(instant),'.fig'],'fig')
end
